package com.qizhen.healsphere.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qizhen.healsphere.repository.mapper.KgCountInfoMapper;
import com.qizhen.healsphere.repository.mapper.KgCountTotalInfoMapper;
import com.qizhen.healsphere.repository.mapper.entity.KgCountInfo;
import com.qizhen.healsphere.repository.mapper.entity.KgCountTotalInfo;
import com.qizhen.healsphere.service.KgCountService;
import com.qizhen.healsphere.web.param.KgCountListPageQuery;
import com.qizhen.healsphere.web.vo.CountInfoVO;
import com.qizhen.healsphere.web.vo.CountListVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 */
@Slf4j
@Service
public class KgCountServiceImpl implements KgCountService {

    private static final Map<String, String> NODE_TYPE_MAP = new HashMap<>();
    
    static {
        NODE_TYPE_MAP.put("Food", "食品");
        NODE_TYPE_MAP.put("Department", "科室");
        NODE_TYPE_MAP.put("Drug", "药品");
        NODE_TYPE_MAP.put("Check", "检查");
        NODE_TYPE_MAP.put("Symptom", "症状");
        NODE_TYPE_MAP.put("Producer", "厂商");
        NODE_TYPE_MAP.put("Disease", "疾病");
    }

    @Autowired
    private KgCountTotalInfoMapper kgCountTotalInfoMapper;

    @Autowired
    private KgCountInfoMapper kgCountInfoMapper;

    @Override
    public CountInfoVO getCountTotalInfo() {
        CountInfoVO countInfoVO = new CountInfoVO();
        List<CountInfoVO.InfoVO> infoVOS = new ArrayList<>();
        
        // 初始化各个统计项
        infoVOS.add(createInfoVO("实体类型", 0));
        infoVOS.add(createInfoVO("实体", 0));
        infoVOS.add(createInfoVO("实体属性", 0));
        infoVOS.add(createInfoVO("关系类型", 0));
        infoVOS.add(createInfoVO("关系", 0));
        infoVOS.add(createInfoVO("术语", 0));

        // 查询统计数据
        KgCountTotalInfo countTotalInfo = kgCountTotalInfoMapper.getCountTotalInfo();

        // 更新统计数据
        if (countTotalInfo != null) {
            infoVOS.get(0).setNum(countTotalInfo.getNodeTypeNum());
            infoVOS.get(1).setNum(countTotalInfo.getNodeNum());
            infoVOS.get(2).setNum(countTotalInfo.getNodePropNum());
            infoVOS.get(3).setNum(countTotalInfo.getRelationTypeNum());
            infoVOS.get(4).setNum(countTotalInfo.getRelationNum());
            infoVOS.get(5).setNum(countTotalInfo.getTermNum());
        }

        countInfoVO.setInfos(infoVOS);
        return countInfoVO;
    }

    private CountInfoVO.InfoVO createInfoVO(String name, Integer num) {
        CountInfoVO.InfoVO infoVO = new CountInfoVO.InfoVO();
        infoVO.setName(name);
        infoVO.setNum(num);
        return infoVO;
    }

    @Override
    public com.qizhen.healsphere.web.vo.Page<CountListVO> getCountListInfo(KgCountListPageQuery query) {
        try {
            Page<KgCountInfo> page = new Page<>(query.getPageNo(), query.getPageSize());
            
            QueryWrapper<KgCountInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("node_num");
            
            Page<KgCountInfo> result = kgCountInfoMapper.selectPage(page, queryWrapper);
            
            if (result == null || result.getRecords() == null) {
                log.warn("查询结果为null");
                return new com.qizhen.healsphere.web.vo.Page<>(
                    query.getPageNo(), 
                    query.getPageSize(), 
                    0, 
                    0, 
                    new ArrayList<>()
                );
            }
            
            // 转换为 VO 对象，并进行类型名称转换
            List<CountListVO> listVOS = result.getRecords().stream()
                .map(info -> {
                    CountListVO vo = new CountListVO();
                    BeanUtil.copyProperties(info, vo);
                    
                    // 转换节点类型名称
                    String nodeType = info.getNodeType();
                    if (nodeType != null) {
                        vo.setNodeType(NODE_TYPE_MAP.getOrDefault(nodeType, nodeType));
                    }
                    
                    return vo;
                })
                .collect(Collectors.toList());
                
            return new com.qizhen.healsphere.web.vo.Page<>(
                query.getPageNo(), 
                query.getPageSize(), 
                listVOS.size(), 
                (int)result.getTotal(), 
                listVOS
            );
        } catch (Exception e) {
            log.error("获取统计列表失败", e);
            throw new RuntimeException("获取统计列表失败", e);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateStatisticsData() {
        try {
            log.info("开始更新统计数据...");
            updateTotalInfo();
            updateDetailInfo();
            log.info("统计数据更新完成");
            return true;
        } catch (Exception e) {
            log.error("更新统计数据失败", e);
            throw new RuntimeException("更新统计数据失败", e);
        }
    }

    private void updateTotalInfo() {
        KgCountTotalInfo totalInfo = kgCountTotalInfoMapper.calculateTotalInfo();
        log.info("计算得到的统计数据: {}", totalInfo);
        if (totalInfo == null) {
            throw new RuntimeException("计算总体统计数据失败");
        }

        KgCountTotalInfo existingInfo = kgCountTotalInfoMapper.getCountTotalInfo();
        log.info("查询到的现有数据: {}", existingInfo);
        
        if (existingInfo != null) {
            totalInfo.setId(existingInfo.getId());
            kgCountTotalInfoMapper.updateById(totalInfo);
            log.info("更新总体统计数据成功");
        } else {
            totalInfo.setId(1L);
            kgCountTotalInfoMapper.insert(totalInfo);
            log.info("插入总体统计数据成功");
        }
    }

    private void updateDetailInfo() {
        try {
            List<KgCountInfo> detailInfoList = kgCountInfoMapper.calculateDetailInfo();
            if (CollUtil.isEmpty(detailInfoList)) {
                log.warn("没有获取到分类统计数据");
                return;
            }

            // 清空现有数据
            kgCountInfoMapper.delete(null);
            log.info("清空分类统计表成功");

            // 批量插入新数据
            for (KgCountInfo info : detailInfoList) {
                kgCountInfoMapper.insert(info);
            }
            log.info("插入分类统计数据成功，共 {} 条记录", detailInfoList.size());
        } catch (Exception e) {
            log.error("更新分类统计数据失败", e);
            throw new RuntimeException("更新分类统计数据失败", e);
        }
    }
}
